<?php

namespace Ecentral\PunchCommerce\Block\Onepage;

use Ecentral\PunchCommerce\Helper\Config;
use Ecentral\PunchCommerce\Helper\ProductMapper;
use Magento\Customer\Model\Session;
use Magento\Quote\Model\Quote\Item;

class Link extends \Magento\Checkout\Block\Onepage\Link
{
    protected Session $customerSession;
    protected ProductMapper $productMappingHelper;
    protected Config $configHelper;

    public function __construct(
        Session $customerSession,
        ProductMapper $productMappingHelper,
        Config $configHelper,
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Checkout\Helper\Data $checkoutHelper,
        array $data = []
    ) {
        $this->customerSession = $customerSession;
        $this->productMappingHelper = $productMappingHelper;
        $this->configHelper = $configHelper;
        parent::__construct($context, $checkoutSession, $checkoutHelper, $data);
    }

    public function isPossibleOnepageCheckout()
    {
        return !$this->isPossiblePunchout() && parent::isPossibleOnepageCheckout();
    }

    public function isPossiblePunchout()
    {
        return $this->configHelper->isActive()
            && $this->customerSession->isLoggedIn()
            && $this->customerSession->getCustomer()->getPunchcommerceCustomerId();
    }

    public function getPunchoutHookUrl()
    {
        return sprintf($this->configHelper->getHookUrlTemplate(), $this->customerSession->getPunchCommerceSid());
    }

    public function getPunchCommerceSid()
    {
        return $this->customerSession->getPunchCommerceSid();
    }

    public function showPunchoutFormFields()
    {
        return $this->configHelper->isDebugMode();
    }

    public function getPunchoutPayload()
    {
        $data = [
            'basket' => [
            ]
        ];

        /** @var Item $item */
        foreach ($this->_checkoutSession->getQuote()->getAllVisibleItems() as $item) {
            $basketItem = [
                'product_ordernumber' => $item->getSku(),
                'product_name'        => $item->getName(),
                'quantity'            => $item->getQty(),
                'item_price'          => (float)$item->getBasePriceInclTax(),
                'price'               => (float)$item->getRowTotalInclTax(),
                'price_net'           => (float)$item->getRowTotal(),
                'tax_rate'            => (int)$item->getTaxPercent(),
                'product'             => $this->productMappingHelper->mapProduct($item)
            ];
            $data['basket'][] = $basketItem;
        }
        return json_encode($data);
    }

    public function getPunchoutButtonLabel()
    {
        return $this->configHelper->getPunchoutButtonLabel();
    }
}
