<?php

namespace Ecentral\PunchCommerce\Controller\Authenticate;

use Ecentral\PunchCommerce\Helper\Cart;
use Ecentral\PunchCommerce\Helper\Config;
use Magento\Customer\Model\ResourceModel\Customer\Collection;
use Magento\Customer\Model\ResourceModel\Customer\CollectionFactory;
use Magento\Customer\Model\Session;

use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Action\HttpGetActionInterface;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\Controller\Result\Redirect;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Message\ManagerInterface;
use Magento\Framework\View\Element\Text;
use Magento\Framework\View\Result\Page;

class Index implements HttpGetActionInterface, HttpPostActionInterface
{
    protected ResultFactory $resultFactory;

    protected Context $context;

    protected Session $customerSession;
    protected \Magento\Checkout\Model\Session $checkoutSession;

    protected CollectionFactory $customerCollectionFactory;
    protected ManagerInterface $messageManager;
    protected Cart $cartHelper;
    private Config $configHelper;

    public function __construct(
        Context $context,
        Session $customerSession,
        CollectionFactory $customerCollectionFactory,
        \Magento\Checkout\Model\Session $checkoutSession,
        ManagerInterface $messageManager,
        Cart $cartHelper,
        Config $configHelper
    ) {
        $this->context = $context;
        $this->customerSession = $customerSession;
        $this->checkoutSession = $checkoutSession;
        $this->customerCollectionFactory = $customerCollectionFactory;
        $this->messageManager = $messageManager;
        $this->cartHelper = $cartHelper;
        $this->configHelper = $configHelper;
    }

    public function execute()
    {
        if (!$this->configHelper->isActive()) {
            $redirect = $this->context->getResultFactory()->create(ResultFactory::TYPE_FORWARD);
            $redirect->setController('index');
            $redirect->forward('defaultNoRoute');
            return $redirect;
        }
        $sId = $this->context->getRequest()->getParam('sId');
        $uId = $this->context->getRequest()->getParam('uId');

        if (empty($sId) || empty($uId)) {
            return $this->prepareErrorResponse('Parameter missing');
        }
        if ($this->customerSession->getPunchCommerceSid($sId) && $this->configHelper->promptBeforeSessionDrop()) {
            if (!$this->context->getRequest()->getParam('relogin')) {
                return $this->context->getResultFactory()->create(ResultFactory::TYPE_PAGE);
            }
        }
        /** @var Collection $customerCollection */
        $customerCollection = $this->customerCollectionFactory->create();
        $customerCollection->addAttributeToFilter('punchcommerce_customer_id', $uId);

        $customer = $customerCollection->getFirstItem();
        if (!$customer->getId()) {
            return $this->prepareErrorResponse('Customer not found!');
        }
        $this->customerSession->setPunchCommerceSid($sId);
        $this->customerSession->setCustomerAsLoggedIn($customer);
        $this->cartHelper->initPunchoutQuote($this->customerSession->getCustomerData());

        /** @var Page $page */
        $page = $this->context->getResultFactory()->create(ResultFactory::TYPE_PAGE);
        $layout = $page->getLayout();

        $redirectUrl = $this->context->getUrl()->getUrl('punchcommerce/authenticate/redirect');
        /** @var Text $text */
        $text = $layout->addBlock(Text::class, 'ecpunchcommerceemptypostform', 'content');
        $text->setText(
            <<<EMPTYFORM
<script type="text/javascript">
require(['jquery','Magento_Customer/js/customer-data'], function($, customerData){
    $('#maincontent').hide();
   customerData.reload(['cart']);
   window.location.href = '$redirectUrl';
});
</script>
EMPTYFORM
        );
        return $page;
    }

    protected function prepareErrorResponse($msg)
    {
        $this->messageManager->addErrorMessage($msg);
        /** @var Redirect $redirect */
        $redirect = $this->context->getResultFactory()->create(ResultFactory::TYPE_REDIRECT);
        $redirect->setUrl($this->context->getRedirect()->getRefererUrl());
        return $redirect;
    }
}
