<?php

namespace Ecentral\PunchCommerce\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Store\Model\ScopeInterface;

class Config extends AbstractHelper
{
    const CONFIG_PATH__ACTIVE = 'ec_punchcommerce/settings/active';
    const CONFIG_PATH__PROMPT_SESSION_DROP = 'ec_punchcommerce/settings/prompt_before_session_drop';
    const CONFIG_PATH__PUNCHOUT_BUTTON_LABEL = 'ec_punchcommerce/settings/punchout_button_label';
    const CONFIG_PATH__FRAME_ANCESTOR = 'ec_punchcommerce/settings/frame_ancestor';
    const CONFIG_PATH__HOOK_URL = 'ec_punchcommerce/cart/hook_url';
    const CONFIG_PATH__PRODUCT_MAPPING = 'ec_punchcommerce/cart/product_mapping';
    const CONFIG_PATH__DEBUG_MODE = 'dev/ec_punchcommerce/debug_mode';

    public function isActive()
    {
        return $this->scopeConfig->getValue(self::CONFIG_PATH__ACTIVE, ScopeInterface::SCOPE_STORE);
    }

    public function getFrameAncestor()
    {
        return trim($this->scopeConfig->getValue(self::CONFIG_PATH__FRAME_ANCESTOR, ScopeInterface::SCOPE_STORE));
    }

    public function getHookUrlTemplate()
    {
        return $this->scopeConfig->getValue(self::CONFIG_PATH__HOOK_URL, ScopeInterface::SCOPE_STORE);
    }

    public function getProductMapping()
    {
        $mappingJson = trim($this->scopeConfig->getValue(
            self::CONFIG_PATH__PRODUCT_MAPPING,
            ScopeInterface::SCOPE_STORE
        ));
        return json_decode($mappingJson, JSON_OBJECT_AS_ARRAY) ??
            throw new \UnexpectedValueException('Product mapping is not valid JSON.');
    }

    public function isDebugMode()
    {
        return $this->scopeConfig->getValue(self::CONFIG_PATH__DEBUG_MODE, ScopeInterface::SCOPE_STORE);
    }

    public function promptBeforeSessionDrop()
    {
        return $this->scopeConfig->getValue(self::CONFIG_PATH__PROMPT_SESSION_DROP, ScopeInterface::SCOPE_STORE);
    }

    public function getPunchoutButtonLabel()
    {
        return $this->scopeConfig->getValue(self::CONFIG_PATH__PUNCHOUT_BUTTON_LABEL, ScopeInterface::SCOPE_STORE);
    }
}
