<?php

namespace Ecentral\PunchCommerce\Setup;

use Magento\Customer\Api\CustomerMetadataInterface;
use Magento\Customer\Model\Customer;
use Magento\Customer\Setup\CustomerSetupFactory;
use Magento\Eav\Model\Entity\Attribute\SetFactory;
use Magento\Framework\Setup\InstallDataInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;

class InstallData implements InstallDataInterface
{
    /**
     * @var CustomerSetupFactory
     */
    private $customerSetupFactory;
    /**
     * @var SetFactory
     */
    private $attributeSetFactory;

    /**
     * InstallData constructor.
     * @param CustomerSetupFactory $customerSetupFactory
     * @param SetFactory $attributeSetFactory
     */
    public function __construct(
        CustomerSetupFactory $customerSetupFactory,
        SetFactory           $attributeSetFactory
    ) {
        $this->customerSetupFactory = $customerSetupFactory;
        $this->attributeSetFactory = $attributeSetFactory;
    }

    /**
     * Installs data for a module
     *
     * @param ModuleDataSetupInterface $setup
     * @param ModuleContextInterface $context
     * @return void
     */
    public function install(ModuleDataSetupInterface $setup, ModuleContextInterface $context)
    {
        $customerSetup = $this->customerSetupFactory->create(['setup' => $setup]);

        $customerSetup->addAttribute(
            Customer::ENTITY,
            'punchcommerce_customer_id',
            [
                'type' => 'varchar',
                'label' => 'PunchCommerce Customer ID',
                'input' => 'text',
                'required' => false,
                'sort_order' => 900,
                'visible' => true,
                'system' => false,
                'position' => 900,
            ]
        );
        $this->setAttributeDefault('punchcommerce_customer_id', $customerSetup);
    }

    /**
     * @param string $attributeCode
     * @param \Magento\Customer\Setup\CustomerSetup $setup
     * @return void
     * @throws \Magento\Framework\Exception\AlreadyExistsException
     */
    public function setAttributeDefault($attributeCode, $setup)
    {
        $setup->updateAttribute(Customer::ENTITY, $attributeCode, 'is_used_in_grid', true);
        $attributeSetId = CustomerMetadataInterface::ATTRIBUTE_SET_ID_CUSTOMER;

        $attributeSet = $this->attributeSetFactory->create();
        $attributeGroupId = $attributeSet->getDefaultGroupId($attributeSetId);

        $usedInForms = [
            'adminhtml_customer',
            'adminhtml_checkout',
            'customer_account_edit',
        ];

        /** @var \Magento\Eav\Model\Entity\Attribute\AbstractAttribute $attribute */
        $attribute = $setup->getEavConfig()->getAttribute(Customer::ENTITY, $attributeCode);
        $attribute->addData([
            'attribute_set_id' => $attributeSetId,
            'attribute_group_id' => $attributeGroupId,
            'used_in_forms' => $usedInForms,
        ]);
        $attribute->getResource()->save($attribute);
    }
}
