<?php

namespace Ecentral\PunchCommerce\Setup;

use Magento\Framework\Setup\InstallSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;

/**
 * Class InstallSchema
 * @package Ecentral\PunchCommerce\Setup
 */
class InstallSchema implements InstallSchemaInterface
{
    /**
     * @param SchemaSetupInterface $setup
     * @param ModuleContextInterface $context
     */
    public function install(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        $installer = $setup;
        $installer->startSetup();

        $this->createCustomerTable($installer);
        $this->createCartMappingOption($installer);

        $installer->endSetup();
    }

    /**
     * Creates the table 'punchcommerce_customer'
     *
     * @param SchemaSetupInterface $installer
     */
    private function createCustomerTable(SchemaSetupInterface $installer): void
    {
        $installer->run("
            CREATE TABLE IF NOT EXISTS `" . $installer->getTable('punchcommerce_customer') . "` (
                `id` int unsigned NOT NULL AUTO_INCREMENT,
                `uid` VARCHAR(255) COLLATE utf8mb4_unicode_ci NOT NULL,
                `last_login` DATETIME(3),
                `created_at` DATETIME(3) NOT NULL,
                `updated_at` DATETIME(3),
                PRIMARY KEY (id),
                UNIQUE `idx.uid` (`uid`)
            )
            ENGINE = InnoDB
            DEFAULT CHARSET = utf8mb4
            COLLATE = utf8mb4_unicode_ci;
            ");
    }
    /**
     * Creates the table 'punchcommerce_cart_mapping_option'
     *
     * @param SchemaSetupInterface $installer
     */
    private function createCartMappingOption(SchemaSetupInterface $installer): void
    {
        $installer->run("
            CREATE TABLE IF NOT EXISTS `" . $installer->getTable('punchcommerce_cart_mapping_option') . "` (
                `id` int unsigned NOT NULL AUTO_INCREMENT,
                `name` VARCHAR(255) COLLATE utf8mb4_unicode_ci NOT NULL UNIQUE,
                `created_at` DATETIME(3) NOT NULL,
                `updated_at` DATETIME(3),
                PRIMARY KEY (id)
            )
            ENGINE = InnoDB
            DEFAULT CHARSET = utf8mb4
            COLLATE = utf8mb4_unicode_ci;
            ");
    }
}
